//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/InterlayerModelItem.h
//! @brief     Defines classes InterlayerModelItem.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2024
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SAMPLE_INTERLAYERMODELITEMS_H
#define BORNAGAIN_GUI_MODEL_SAMPLE_INTERLAYERMODELITEMS_H

#include "Sample/Interface/InterlayerModels.h"
#include <QXmlStreamReader>
#include <memory>

class InterlayerModelItem {
public:
    virtual ~InterlayerModelItem() = default;

    virtual void writeTo(QXmlStreamWriter*) const {}
    virtual void readFrom(QXmlStreamReader*) {}

    virtual std::unique_ptr<InterlayerModel> createModel() const = 0;

protected:
    InterlayerModelItem() = default;
};

class ErfInterlayerItem : public InterlayerModelItem {
public:
    ErfInterlayerItem() = default;
    std::unique_ptr<InterlayerModel> createModel() const
    {
        return std::make_unique<ErfInterlayer>();
    }
};

class TanhInterlayerItem : public InterlayerModelItem {
public:
    TanhInterlayerItem() = default;
    std::unique_ptr<InterlayerModel> createModel() const
    {
        return std::make_unique<TanhInterlayer>();
    }
};

#endif // BORNAGAIN_GUI_MODEL_SAMPLE_INTERLAYERMODELITEMS_H
