//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/RoughnessCatalog.cpp
//! @brief     Implements class RoughnessCatalog.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2022
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Sample/RoughnessCatalog.h"
#include "Base/Util/Assert.h"
#include "GUI/Model/Sample/InterlayerModelItems.h"
#include "GUI/Model/Sample/RoughnessItems.h"

RoughnessItem* RoughnessCatalog::create(Type type)
{
    switch (type) {
    case Type::None:
        return nullptr;
    case Type::Basic:
        return new BasicRoughnessItem(0., .3, 5.);
    }
    ASSERT_NEVER;
}

QVector<RoughnessCatalog::Type> RoughnessCatalog::types()
{
    return {Type::None, Type::Basic};
}

UiInfo RoughnessCatalog::uiInfo(Type type)
{
    switch (type) {
    case Type::None:
        return {"None", "", ""};
    case Type::Basic:
        return {"Basic", "Fractal lateral correlation function", ""};
    }
    ASSERT_NEVER;
}

RoughnessCatalog::Type RoughnessCatalog::type(const RoughnessItem* item)
{
    if (!item)
        return Type::None;

    if (dynamic_cast<const BasicRoughnessItem*>(item))
        return Type::Basic;

    ASSERT_NEVER;
}

//--------------------------------------------------------------------------

InterlayerModelItem* InterlayerModelCatalog::create(Type type)
{
    switch (type) {
    case Type::Erf:
        return new ErfInterlayerItem();
    case Type::Tanh:
        return new TanhInterlayerItem();
    }
    ASSERT_NEVER;
}

QVector<InterlayerModelCatalog::Type> InterlayerModelCatalog::types()
{
    return {Type::Erf, Type::Tanh};
}

UiInfo InterlayerModelCatalog::uiInfo(Type type)
{
    switch (type) {
    case Type::Erf:
        return {"Erf", "Interlayer profile is Erf function", ""};
    case Type::Tanh:
        return {"Tanh", "Interlayer profile is Tanh function", ""};
    }
    ASSERT_NEVER;
}

InterlayerModelCatalog::Type InterlayerModelCatalog::type(const InterlayerModelItem* model)
{
    if (dynamic_cast<const ErfInterlayerItem*>(model))
        return Type::Erf;

    if (dynamic_cast<const TanhInterlayerItem*>(model))
        return Type::Tanh;

    ASSERT_NEVER;
}
